/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package clientwizard;

import eu.medsea.mimeutil.MimeUtil;
import java.io.File;
import java.io.RandomAccessFile;
import java.net.NetworkInterface;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.Stack;

/**
 *
 * @author idrago
 */
public class FileTable extends Thread {
    static private final String NL = "\n";
    static private final String SEP = " ";
    
    static MessageDigest digest;
    static private final int SIZE_HASH = 8*1024;
        
    static private final int DIM = 10; /* Dimension */
    static private final int SIZE = 0;
    static private final int TIME = 1;
    static private final int MIME = 2;
    static private final int EXT  = 3;
    static private final int NAME = 4;
    static private final int BMD5 = 5;
    static private final int EMD5 = 6;
    static private final int FILE = 7;
    static private final int PARENT = 8;
    static private final int FULLPATH = 9;
    
    public ArrayList<String[]> log;   
    
    public String uID = null;
    public String start_time;
    public String end_time;
    private long files;
    private long folders;
    private long bytes;
    
    private Wizard parent;
    private GetMimeTypes mime = new GetMimeTypes(); 
    
    public FileTable(Wizard parent) {
        this.parent = parent;
        
        log = new ArrayList<String[]>();   
        try {
            digest = MessageDigest.getInstance("MD5");
        } catch (Exception e) { 
            parent.error_log.append("FileTable ").append(e.getMessage()).append(";");
            digest = null;            
        }
        files = 0;
        folders = 1;
        bytes = 0;
    }
    
    public int getSize() {
        if (log != null) { 
            return log.size();
        } else { 
            return 0;
        }
    } 
    
    private String prepareMessage() {        
        byte[] folder = parent.dropbox_folder.getAbsolutePath().getBytes();
        
        StringBuilder sb = new StringBuilder();
        sb.append("start ").append(start_time).append(NL);
        sb.append("end ").append(end_time).append(NL);
        sb.append("id ").append(uID).append(NL);            
        sb.append("folder ").append(getHash(folder, folder.length)).append(SEP);
        sb.append("default ").append(parent.def_folder).append(SEP);
        sb.append(parent.found_folder).append(SEP);
        sb.append(parent.match_folder).append(SEP);
        sb.append(System.getProperty("os.name")).append(SEP);
        sb.append(Locale.getDefault()).append(SEP);
        sb.append(Charset.defaultCharset()).append(NL);
        sb.append("Errors: ").append(parent.error_log).append(NL);
        sb.append("file size time mime extension first8k_md5 last8k_md5 name_md5 parent fullpath").append(NL);
        for (String[] s : log) {
            sb.append(s[FILE]).append(SEP);
            sb.append(s[SIZE]).append(SEP);
            sb.append(s[TIME]).append(SEP);
            sb.append(s[MIME]).append(SEP);
            sb.append(s[EXT]).append(SEP);
            sb.append(s[BMD5]).append(SEP);            
            sb.append(s[EMD5]).append(SEP);            
            sb.append(s[NAME]).append(SEP);
            sb.append(s[PARENT]).append(SEP);
            sb.append(s[FULLPATH]).append(NL);
        }        
        return sb.toString();
    }    
    
    public String getMessage() {
        return prepareMessage();
    }
    
    public String getObject(int x, int y) {
        if (log != null && (log.size() > x)) { 
            if (y == SIZE) { 
                long v = Long.valueOf(log.get(x)[SIZE])/1024;
                if (v < 1024) {
                    return String.valueOf(v) + "KB";   
                } else { 
                    return String.valueOf(v/1024) + "MB";   
                }
            } else {
                return log.get(x)[y];   
            }
        } else { 
            return "-";
        }
    }    
        
    public void run() {
        getFiles();
    }
    
    private static String getHashString(MessageDigest digest) {
        byte[] md =  digest.digest();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < md.length; i++) {
            sb.append(Integer.toString((md[i] & 0xff) + 0x100, 16).substring(1));
        }
        return sb.toString();
    }        
    
    private static String getHash(byte[] myBytes, int len) {
        try {
            digest.reset();
            if (len > 0)  {
                digest.update(myBytes, 0, len);
            }
            return getHashString(digest);
        }
        catch (Exception e) {            
            return "md5_error";
        }
    }    
    
    private String getUserIDConfig() {
        int read;
        byte[] buffer = new byte[SIZE_HASH];
        try {
            if (parent.HOSTKEYS != null) {
                RandomAccessFile ris = new RandomAccessFile(parent.HOSTKEYS, "r");
                read = ris.read(buffer);
                ris.close();
                if (read > 0) {
                    return getHash(buffer, read);
                }                
            }
        } catch (Exception e) {
            parent.error_log.append("getUserIDConfig ").append(e.getMessage()).append(";");
        }
        return null;
    }    
    
    private String getUserIDMAC() {
        int c = 0;
        List<byte[]> macs = new ArrayList<byte[]>();
	try {
            for (Enumeration e = NetworkInterface.getNetworkInterfaces(); e.hasMoreElements() ;) {
                NetworkInterface n = (NetworkInterface)e.nextElement();
                try {
                    macs.add(n.getHardwareAddress());
                } catch (NullPointerException ex) {
                    // Fine - localhost does not have mac
                }
            }
        } catch (Exception ex) {
            parent.error_log.append("getUserIDMAC ").append(ex.getMessage()).append(";");
        }
        digest.reset();
        for (byte[] m : macs)  {
            if (m != null) {
                digest.update(m, 0, m.length);
                c++;
            }
        }
        if (c > 0) { 
            return getHashString(digest);
        } else { 
            return null;
        }
    }    
        
    private String getUserID() {
        String id;
        
        id = getUserIDConfig();
        if (id != null && id.length() > 0) { 
            return "cf_" + id;
        } 
        id = getUserIDMAC();
        if (id != null && id.length() > 0) { 
            return "ma_" + id;
        }      
        return "NULL_ID";
    }
    
    private void getFiles() {
        Stack<File> stack = new Stack<File>();
        start_time = Long.toString(new Date().getTime());
        uID = getUserID();
        
        stack.push(parent.dropbox_folder);
        while (!stack.isEmpty()) {
            File par_dir = stack.pop();
            String par_path = par_dir.getPath();
            File[] content = par_dir.listFiles();
            if (content != null) { 
                for (File file : content) {
                    String[] new_file = new String[DIM];
                    new_file[PARENT] = getHash(par_path.getBytes(), par_path.getBytes().length); 
                    new_file[FULLPATH] = "-"; 
                            
                    String name = file.getName();
                    bytes += file.length();
                    byte[] mName = name.getBytes();
                    new_file[NAME] = getHash(mName, mName.length);
                    new_file[SIZE] = Long.toString(file.length());
                    new_file[TIME] = Long.toString(file.lastModified());
                    if (file.isFile()) { 
                        new_file[FILE] = "1";   
                    } else {
                        new_file[FILE] = "0";   
                        new_file[FULLPATH] = getHash(file.getPath().getBytes(), file.getPath().getBytes().length); 
                    }
                    new_file[EXT] = "-";
                    new_file[BMD5] = "-";    
                    new_file[EMD5] = "-";
                    new_file[MIME] = "[application/directory]";
                    
                    if (file.isFile() && file.canRead()) {
                        //if file name has at least one dot (and the last dot is not the first letter)                        
                        if ((name.lastIndexOf('.') > 0) && (name.lastIndexOf('.') < name.length() - 1)) {
                            new_file[EXT] = name.substring(name.lastIndexOf('.') + 1); 
                            new_file[EXT] = new_file[EXT].replaceAll(" ", "_");
                        }                        
                        try {
                            byte[] buffer = new byte[SIZE_HASH];
                            /* read some bytes of the file */
                            RandomAccessFile ris = new RandomAccessFile(file, "r");
                            int read = ris.read(buffer);
                            new_file[BMD5] = getHash(buffer, read);
                            new_file[MIME] = mime.GetTypes(buffer, file.getAbsolutePath());
                            
                            if (file.length() > SIZE_HASH) {
                                ris.seek(file.length() - SIZE_HASH);
                                read = ris.read(buffer);
                                new_file[EMD5] = getHash(buffer, read);
                            }
                            ris.close();
                        } catch (Exception e) {
                            parent.error_log.append("getFiles ").append(e.getMessage()).append(";");
                        }
                        files++;
                        if (files % 50 == 0) {
                            parent.panel_wz2.refresh(files, folders, bytes);
                        }                        
                    } else if (file.isDirectory() && file.canRead()) {                                                 
                        stack.push(file);                        
                        folders++;
                        if (folders % 20 == 0) {
                            parent.panel_wz2.refresh(files, folders, bytes);
                        }
                    } else {
                        if (!file.canRead()) {
                            // System.out.println(file.getAbsolutePath());
                            parent.error_log.append("Error: ").append(file.canRead()).append(";");
                        } else {
                            parent.error_log.append("Neither a file nor a folder;");
                        }                        
                    }
                    log.add(new_file);
                }
            } else {
                parent.error_log.append("I got an empty folder;");
            }
        }
        parent.panel_wz2.refresh(files, folders, bytes);
        end_time = Long.toString(new Date().getTime());        
        parent.panel_wz2.stop();
    }  
    
    private class GetMimeTypes {
        public GetMimeTypes() {
            MimeUtil.unregisterMimeDetector("eu.medsea.mimeutil.detector.MagicMimeMimeDetector");
            MimeUtil.unregisterMimeDetector("eu.medsea.mimeutil.detector.ExtensionMimeDetector"); 
            MimeUtil.registerMimeDetector("eu.medsea.mimeutil.detector.MagicMimeMimeDetector"); 
            MimeUtil.registerMimeDetector("eu.medsea.mimeutil.detector.ExtensionMimeDetector");        
        }
        
        public Collection GetTypesCollection(String filename)  {   
            return MimeUtil.getMimeTypes(filename);
        }        
        
        public Collection GetTypesCollection(byte[] sample)  {   
            return MimeUtil.getMimeTypes(sample);
        }
        
        public String GetTypes(byte[] sample, String filename)  {   
            Collection c1, c2;
            c1 = GetTypesCollection(filename);
            String a; 
            if (c1 !=  null) {
                a = MimeUtil.getMostSpecificMimeType(c1).toString();
                if (!a.equals("null")) {
                    return "[" + a.replaceAll(" ", "_") + "]"; 
                }                
            }
            c2 = GetTypesCollection(sample);
            if (c2 !=  null) {
                a = MimeUtil.getMostSpecificMimeType(c2).toString();
                if (!a.equals("null")) {
                    return "[" + a.replaceAll(" ", "_") + "]"; 
                }                
            }       
            return "[]";
        }
    }  
}
